import json
from sqlalchemy.orm import class_mapper
from sqlalchemy.exc import DatabaseError
from .exception import CoreException
from .app import db


class Entity(object):
	__json_public__ = None
	__json_hidden__ = None
	__json_modifiers__ = None
	
	def __init__(self, obj):
		"""
		:rtype: object
		"""
		self.__mapper__ = None
		self.from_dict(self, obj)
		pass
	
	@staticmethod
	def from_dict(self, obj):
		for k, v in obj.iteritems():
			if isinstance(v, dict):
				setattr(self, k, Entity(v))
			else:
				setattr(self, k, v)
	
	def save(self):
		db.session.add(self)
		self.flush()
	
	def delete(self):
		db.session.delete(self)
		self.flush()
	
	def update(self, obj):
		if obj is None:
			raise CoreException('object cannot be none')
		if not isinstance(obj, dict):
			raise CoreException('object must be instances of dict')
		for attr, value in obj.items():
			print ('{}:{}'.format(attr, value))
			setattr(self, attr, value)
		return self.save()
	
	@staticmethod
	def flush():
		try:
			db.session.flush()
		except DatabaseError:
			db.session.rollback()
			raise
	
	def get_field_names(self):
		for p in class_mapper(self.__class__).iterate_properties:
			yield p.key
	
	def to_dict(self):
		field_names = self.get_field_names()
		public = self.__json_public__ or field_names
		hidden = self.__json_hidden__ or []
		modifiers = self.__json_modifiers__ or dict()
		rv = dict()
		for key in public:
			value = getattr(self, key)
			if isinstance(value, list):
				child_list = []
				for v in value:
					child = {}
					for k in class_mapper(v.__class__).iterate_properties:
						child[k.key] = getattr(v, k.key)
					child_list.append(child)
				if len(child_list) > 0:
					rv.update({key: child_list})
				continue
			
			if issubclass(type(value), db.Model):
				child = {}
				for k in class_mapper(value.__class__).iterate_properties:
					child[k.key] = getattr(value, k.key)
					pass
				rv.update({key: child})
				continue
			
			if value is not None:
				rv[key] = value
		for key, modifier in modifiers.items():
			value = getattr(self, key)
			rv[key] = modifier(value, self)
		for key in hidden:
			rv.pop(key, None)
		return rv
	
	def to_json(self):
		return json.dump(self.to_dict())



# from unittest import TestCase
#
#
# class TestValidator(TestCase):
# 	service = None
#
# 	def setUp(self):
# 		self.service = Service()
#
# 	def test_keys_fail(self):
# 		try:
# 			self.service.key_service({'user': {'name':'hell', 'alamat':'Jakarta'}})
# 		except ValidationException as e:
# 			self.assertEqual(e.key, 'nama')
# 			self.assertEqual(e.message, 'required.key')
#
# 	def test_keys_success(self):
# 		domain = self.service.key_service({'user': [{'nama': 'hell', 'alamat':''}, {'nama': 'hell', 'alamat':''}]})
# 		self.assertTrue('user' in domain)
# 		for data in domain['user']:
# 			self.assertTrue('nama' in data)
#
# 	def test_empty(self):
# 		try:
# 			self.service.value_service({'user': {'nama': ''}})
# 		except ValidationException as e:
# 			self.assertEqual(e.message, 'value.cannot.be.empty')
#
# 	def test_empty_success(self):
# 		domain = self.service.value_service({'user': {'nama': 'Rifky'}})
# 		self.assertTrue('user' in domain)
# 		self.assertTrue('nama' in domain['user'])
#
# 	def test_number_fail(self):
# 		try:
# 			self.service.number_service({'trx': {'trx_id': 'data'}})
# 		except ValidationException as e:
# 			self.assertEqual(e.message, 'required.number.trx_id')
# 			self.assertEqual(e.key, 'trx_id')
#
# 	def test_mail_fail(self):
# 		try:
# 			self.service.mail_service({'user': {'email': 'data'}})
# 		except ValidationException as e:
# 			self.assertEqual(e.message, 'invalid.email')
# 			self.assertEqual(e.key, 'email')
#
# 	def test_phone_fail(self):
# 		try:
# 			self.service.phone_service({'user': {'name': 'kiditz', 'phone': 'sa'}, 'alamat':'as'})
# 		except ValidationException as e:
# 			self.assertEqual(e.message, 'invalid.phone.number')
#
#
# class Service(object):
#
# 	@Key(['user.nama.alamat'])
# 	def key_service(self, domain):
# 		return domain
#
# 	@Email(['user.email'])
# 	def mail_service(self, domain):
# 		return domain
#
# 	@Empty(['user.nama'])
# 	def value_service(self, domain):
# 		return domain
#
# 	@Number(['trx.trx_id'])
# 	def number_service(self, domain):
# 		return domain
#
# 	@Blank(['user.name.phone', 'alamat'])
# 	@Phone(['user.phone'])
# 	def phone_service(self, domain):
# 		return domain


import re
from slerp import is_empty, is_not_digit, is_blank
from .exception import ValidationException,CoreException


class Key(object):
	def __init__(self, keys):
		if not isinstance(keys, list):
			raise TypeError('keys.should.be.instance.of.list')
		self.keys = keys
	
	def __call__(self, original_function):
		def new_function(other, domain):
			if not isinstance(domain, dict):
				raise ValidationException('input.must.domain')
			for key in self.keys:
				if '.' in key:
					split = key.split('.')
					if split[0] not in domain:
						raise ValidationException('required.key', key=split[0])
					if isinstance(domain[split[0]], list):
						for k in split[1:]:
							for i, d in enumerate(domain[split[0]]):
								if k not in d:
									raise ValidationException('required.key', key='{}[{}]'.format(k, i))
					elif isinstance(domain[split[0]], dict):
						for k in split[1:]:
							if k not in domain[split[0]]:
								raise ValidationException('required.key', key='{}'.format(k))
					else:
						raise CoreException('object.type.unknown')
				else:
					if key not in domain:
						raise ValidationException('required.key', key=key)
			return original_function(other, domain)
		
		return new_function


class Empty(object):
	def __init__(self, keys):
		if not isinstance(keys, list):
			raise TypeError('keys.should.be.instance.of.list')
		self.keys = keys
		pass
	
	def __call__(self, original_function):
		@Key(self.keys)
		def new_function(other, domain):
			for key in self.keys:
				if '.' in key:
					split = key.split('.')
					if isinstance(domain[split[0]], list):
						for k in split[1:]:
							for i, d in enumerate(domain[split[0]]):
								if is_empty(d[k]):
									raise ValidationException('value.cannot.be.empty', key='{}[{}]'.format(k, i))
					elif isinstance(domain[split[0]], dict):
						for k in split[1:]:
							d = domain[split[0]]
							for k1 in d:
								if is_empty(d[k1]):
									raise ValidationException('value.cannot.be.empty', key='{}'.format(k))
					else:
						raise CoreException('object.type.unknown')
				else:
					if is_empty(domain[key]):
						raise ValidationException('value.cannot.be.empty', key=key)
			return original_function(other, domain)
		
		return new_function


class Blank(object):
	def __init__(self, keys):
		if not isinstance(keys, list):
			raise TypeError('keys.should.be.instance.of.list')
		self.keys = keys
		pass
	
	def __call__(self, original_function):
		@Key(self.keys)
		def new_function(other, domain):
			for key in self.keys:
				if '.' in key:
					split = key.split('.')
					if isinstance(domain[split[0]], list):
						for k in split[1:]:
							for i, d in enumerate(domain[split[0]]):
								if is_blank(d[k]):
									raise ValidationException('value.cannot.be.blank', key='{}[{}]'.format(k, i))
					elif isinstance(domain[split[0]], dict):
						for k in split[1:]:
							d = domain[split[0]]
							for k1 in d:
								if is_blank(d[k1]):
									raise ValidationException('value.cannot.be.blank', key='{}'.format(k))
					else:
						raise CoreException('object.type.unknown')
				else:
					if is_blank(domain[key]):
						raise ValidationException('value.cannot.be.blank', key=key)
			return original_function(other, domain)
		
		return new_function


class Number(object):
	def __init__(self, keys):
		if not isinstance(keys, list):
			raise ValidationException('keys.should.be.instance.of.list')
		self.keys = keys
		pass
	
	def __call__(self, original_function):
		@Key(self.keys)
		def new_function(other, domain):
			
			for key in self.keys:
				if '.' in key:
					split = key.split('.')
				
					if isinstance(domain[split[0]], list):
						for k in split[1:]:
							for i, d in enumerate(domain[split[0]]):
								if is_not_digit(d[k]):
									raise ValidationException('required.number.{}'.format(k), key='{}[{}]'.format(k, i))
					elif isinstance(domain[split[0]], dict):
						for k in split[1:]:
							d = domain[split[0]]
							for k1 in d:
								if is_not_digit(d[k1]):
									raise ValidationException('required.number.{}'.format(k), key='{}'.format(k))
					else:
						raise CoreException('object.type.unknown')
				else:
					if is_not_digit(str(domain[key])):
						raise ValidationException('required.number.' + key, key=key)
			return original_function(other, domain)
		
		return new_function


class Email(object):
	
	def __init__(self, keys):
		if not isinstance(keys, list):
			raise ValidationException('keys.should.be.instance.of.list')
		self.keys = keys
		self.mail_re = r"^[_A-Za-z0-9-\+]+(\.[_A-Za-z0-9-]+)*@[A-Za-z0-9-]+(\.[A-Za-z0-9]+)*(\.[A-Za-z]{2,})$"
		pass
	
	def __call__(self, original_function):
		@Key(self.keys)
		def new_function(other, domain):
			for key in self.keys:
				if '.' in key:
					split = key.split('.')
					if isinstance(domain[split[0]], list):
						for k in split[1:]:
							for i, d in enumerate(domain[split[0]]):
								if not bool(re.match(self.mail_re, d[k])):
									raise ValidationException('invalid.email', key='{}[{}]'.format(k, i))
					elif isinstance(domain[split[0]], dict):
						for k in split[1:]:
							d = domain[split[0]]
							for k1 in d:
								if not bool(re.match(self.mail_re, d[k1])):
									raise ValidationException('invalid.email', key='{}'.format(k))
					else:
						raise CoreException('object.type.unknown')
				else:
					if not bool(re.match(self.mail_re, domain[key])):
						raise ValidationException('invalid.email', key=key)
			return original_function(other, domain)
		return new_function


class Phone(object):
	def __init__(self, keys):
		if not isinstance(keys, list):
			raise ValidationException('keys.should.be.instance.of.list')
		self.keys = keys
		self.phone_re = r"(\+[0-9]+[\- \.]*)?(\([0-9]+\)[\- \.]*)?([0-9][0-9\-\.]+[0-9])"
		pass
	
	def __call__(self, original_function):
		@Key(self.keys)
		def new_function(other, domain):
			if not isinstance(domain, dict):
				raise ValidationException('input.must.domain')
			for key in self.keys:
				if '.' in key:
					split = key.split('.')
					if isinstance(domain[split[0]], list):
						for k in split[1:]:
							for i, d in enumerate(domain[split[0]]):
								if not bool(re.match(self.phone_re, d[k])):
									raise ValidationException('invalid.phone.number.{}'.format(k), key='{}[{}]'.format(k, i))
					elif isinstance(domain[split[0]], dict):
						for k in split[1:]:
							d = domain[split[0]]
							for k1 in d:
								if not bool(re.match(self.phone_re, d[k1])):
									raise ValidationException('invalid.phone.number', key='{}'.format(k))
					else:
						raise CoreException('object.type.unknown')
				else:
					if not bool(re.match(self.phone_re, domain[key])):
						raise ValidationException('invalid.phone.number', key=key)
			return original_function(other, domain)
		return new_function





import logging
import sys
import os
import json
from logging.config import dictConfig
import codecs

DEFAULT_CONFIG = 'config.json'

if not os.path.exists(DEFAULT_CONFIG):
	raise RuntimeError('please create config.json file in your project root folder')

syslog_format = ("[%(name)s][env:{logging_env}] %(levelname)s ""[{hostname}  %(process)d] [%(filename)s:%(lineno)d] ""- %(message)s").format(logging_env="", hostname="")
json_file = open(DEFAULT_CONFIG, 'rb')
reader = codecs.getreader('utf-8')
configurator = json.load(reader(json_file))
json_file.close()

logging.getLogger('werkzeug').setLevel(logging.DEBUG)
logging.getLogger('socketIO-client').setLevel(logging.ERROR)
logging.getLogger('apscheduler.executors.default').setLevel(logging.ERROR)

LOGGING = {
	'version': 1,
	'disable_existing_loggers': False,
	'formatters': {
		'standard': {
			'format': '%(asctime)s %(levelname)s %(process)d '
			          '[%(name)s] %(filename)s:%(lineno)d - %(message)s',
		},
		'syslog_format': {'format': syslog_format},
		'raw': {'format': '%(message)s'},
	},
	'handlers': {
		'console': {
			'level': 'INFO',
			'class': 'logging.StreamHandler',
			'formatter': 'standard',
			'stream': sys.stdout,
		},
		'rotate_file': {
			'level': 'DEBUG',
			'formatter': 'standard',
			'class': 'logging.handlers.RotatingFileHandler',
			'filename': configurator['log_file'],
			'encoding': 'utf8',
			'maxBytes': 100000,
			'backupCount': 1,
		}
	},
	'loggers': {
		'requests.packages.urllib3.connectionpool': {
			'handlers': [],
			'propagate': False,
			'level': 'INFO',
		},
		'': {
			'handlers': ['console', 'rotate_file'],
			'level': 'DEBUG',
			'propagate': False
		},
	}
}

dictConfig(LOGGING)



# coding: utf-8

import time
import functools

from requests.exceptions import ConnectionError, ConnectTimeout

from .exception import ConsulException


def with_retry_connections(max_tries=3, sleep=0.05):
    """
    Decorator that wraps an entire function in a try/except clause. On
    requests.exceptions.ConnectionError, will re-run the function code
    until success or max_tries is reached.

    :param max_tries: maximum number of attempts before giving up
    :param sleep: time to sleep between tries, or None
    """
    def decorator(f):
        @functools.wraps(f)
        def f_retry(*args, **kwargs):
            tries = 0
            while True:
                try:
                    return f(*args, **kwargs)
                except (ConnectionError, ConnectTimeout) as e:
                    tries += 1
                    if tries >= max_tries:
                        raise ConsulException(e)
                    if sleep:
                        time.sleep(sleep)
        return f_retry
    return decorator


import json
import random
import re
import unicodedata
from uuid import uuid4
import string
URL_RE = re.compile(
	r'^'
	r'([a-z-]+://)'  # scheme
	r'([a-z_\d-]+:[a-z_\d-]+@)?'  # user:password
	r'(www\.)?'  # www.
	r'((?<!\.)[a-z\d]+[a-z\d\.-]+\.[a-z]{2,6}|\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}|localhost)'  # domain
	r'(:\d{2,})?'  # port number
	r'(/[a-z\d_%\+-]*)*'  # folders
	r'(\.[a-z\d_%\+-]+)*'  # file extension
	r'(\?[a-z\d_\+%-=]*)?'  # query string
	r'(#\S*)?'  # hash
	r'$',
	re.IGNORECASE
)
EMAIL_RE = re.compile(r'^[a-zA-Z\d\._\+-]+@([a-z\d-]+\.?[a-z\d-]+)+\.[a-z]{2,4}$')
CAMEL_CASE_TEST_RE = re.compile(r'^[a-zA-Z]*([a-z]+[A-Z]+|[A-Z]+[a-z]+)[a-zA-Z\d]*$')
CAMEL_CASE_REPLACE_RE = re.compile(r'([a-z]|[A-Z]+)(?=[A-Z])')
SNAKE_CASE_TEST_RE = re.compile(r'^[a-z]+([a-z\d]+_|_[a-z\d]+)+[a-z\d]+$')
SNAKE_CASE_TEST_DASH_RE = re.compile(r'^[a-z]+([a-z\d]+-|-[a-z\d]+)+[a-z\d]+$')
SNAKE_CASE_REPLACE_RE = re.compile(r'(_)([a-z\d])')
SNAKE_CASE_REPLACE_DASH_RE = re.compile('(-)([a-z\d])')
CREDIT_CARDS = {
	'VISA': re.compile(r'^4[0-9]{12}(?:[0-9]{3})?$'),
	'MASTERCARD': re.compile(r'^5[1-5][0-9]{14}$'),
	'AMERICAN_EXPRESS': re.compile(r'^3[47][0-9]{13}$'),
	'DINERS_CLUB': re.compile(r'^3(?:0[0-5]|[68][0-9])[0-9]{11}$'),
	'DISCOVER': re.compile(r'^6(?:011|5[0-9]{2})[0-9]{12}$'),
	'JCB': re.compile(r'^(?:2131|1800|35\d{3})\d{11}$')
}
JSON_WRAPPER_RE = re.compile(r'^\s*\{\s*.*\s*\}\s*$', re.MULTILINE | re.DOTALL)
UUID_RE = re.compile(r'^[a-f\d]{8}-[a-f\d]{4}-[a-f\d]{4}-[a-f\d]{4}-[a-f\d]{12}$', re.IGNORECASE)
IP_RE = re.compile(r'^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$')
WORDS_COUNT_RE = re.compile(r'\W*[^\W_]+\W*', re.IGNORECASE | re.MULTILINE | re.UNICODE)
HTML_RE = re.compile(
	r'((<([a-z]+:)?[a-z]+[^>]*/?>)(.*?(</([a-z]+:)?[a-z]+>))?|<!--.*-->|<!doctype.*>)',
	re.IGNORECASE | re.MULTILINE | re.DOTALL
)
HTML_TAG_ONLY_RE = re.compile(
	r'(<([a-z]+:)?[a-z]+[^>]*/?>|</([a-z]+:)?[a-z]+>|<!--.*-->|<!doctype.*>)',
	re.IGNORECASE | re.MULTILINE | re.DOTALL
)
PRETTIFY_RE = {
	# match repetitions of signs that should not be repeated (like multiple spaces or duplicated quotes)
	'DUPLICATES': re.compile(
		r'(\({2,}|\){2,}|\[{2,}|\]{2,}|\{{2,}|\}{2,}|:{2,}|,{2,}|;{2,}|\+{2,}|\-{2,}|\s{2,}|%{2,}|={2,}|"{2,}|\'{2,})',
		re.MULTILINE
	),
	# check that a sign cannot have a space before or missing a space after,
	# unless it is a dot or a comma, where numbers may follow (5.5 or 5,5 is ok)
	'RIGHT_SPACE': re.compile(
		r'('
		r'(?<=[^\s\d]),(?=[^\s\d])|\s,\s|\s,(?=[^\s\d])|\s,(?!.)|'  # comma (,)
		r'(?<=[^\s\d\.])\.+(?=[^\s\d\.])|\s\.+\s|\s\.+(?=[^\s\d])|\s\.+(?!\.)|'  # dot (.)
		r'(?<=\S);(?=\S)|\s;\s|\s;(?=\S)|\s;(?!.)|'  # semicolon (;)
		r'(?<=\S):(?=\S)|\s:\s|\s:(?=\S)|\s:(?!.)|'  # colon (:)
		r'(?<=[^\s!])!+(?=[^\s!])|\s!+\s|\s!+(?=[^\s!])|\s!+(?!!)|'  # exclamation (!)
		r'(?<=[^\s\?])\?+(?=[^\s\?])|\s\?+\s|\s\?+(?=[^\s\?])|\s\?+(?!\?)|'  # question (?)
		r'\d%(?=\S)|(?<=\d)\s%\s|(?<=\d)\s%(?=\S)|(?<=\d)\s%(?!.)'  # percentage (%)
		r')',
		re.MULTILINE | re.DOTALL
	),
	'LEFT_SPACE': re.compile(
		r'('
		
		# quoted text ("hello world")
		r'\s"[^"]+"(?=[\?\.:!,;])|(?<=\S)"[^"]+"\s|(?<=\S)"[^"]+"(?=[\?\.:!,;])|'
		
		# text in round brackets
		r'\s\([^\)]+\)(?=[\?\.:!,;])|(?<=\S)\([^\)]+\)\s|(?<=\S)(\([^\)]+\))(?=[\?\.:!,;])'
		
		r')',
		re.MULTILINE | re.DOTALL
	),
	# match chars that must be followed by uppercase letters (like ".", "?"...)
	'UPPERCASE_AFTER_SIGN': re.compile(
		r'([\.\?!]\s\w)',
		re.MULTILINE | re.UNICODE
	),
	'SPACES_AROUND': re.compile(
		r'('
		r'(?<=\S)\+(?=\S)|(?<=\S)\+\s|\s\+(?=\S)|'  # plus (+)
		r'(?<=\S)\-(?=\S)|(?<=\S)\-\s|\s\-(?=\S)|'  # minus (-)
		r'(?<=\S)/(?=\S)|(?<=\S)/\s|\s/(?=\S)|'  # division (/)
		r'(?<=\S)\*(?=\S)|(?<=\S)\*\s|\s\*(?=\S)|'  # multiplication (*)
		r'(?<=\S)=(?=\S)|(?<=\S)=\s|\s=(?=\S)|'  # equal (=)
		
		# quoted text ("hello world")
		r'\s"[^"]+"(?=[^\s\?\.:!,;])|(?<=\S)"[^"]+"\s|(?<=\S)"[^"]+"(?=[^\s\?\.:!,;])|'
		
		# text in round brackets
		r'\s\([^\)]+\)(?=[^\s\?\.:!,;])|(?<=\S)\([^\)]+\)\s|(?<=\S)(\([^\)]+\))(?=[^\s\?\.:!,;])'
		
		r')',
		re.MULTILINE | re.DOTALL
	),
	'SPACES_INSIDE': re.compile(
		r'('
		r'(?<=")[^"]+(?=")|'  # quoted text ("hello world")
		r'(?<=\()[^\)]+(?=\))'  # text in round brackets
		r')',
		re.MULTILINE | re.DOTALL
	),
	'SAXON_GENITIVE': re.compile(
		r'('
		r'(?<=\w)\'\ss\s|(?<=\w)\s\'s(?=\w)|(?<=\w)\s\'s\s(?=\w)'
		r')',
		re.MULTILINE | re.UNICODE
	)
}
NO_LETTERS_OR_NUMBERS_RE = re.compile(r'[^\w\d]+|_+', re.IGNORECASE | re.UNICODE)
SPACES_RE = re.compile(r'\s')

letters_set = set('abcdefghijklmnopqrstuvwxyz')


# string checking functions


def is_string(obj):
	"""
    Checks if an object is a string.

    :param obj: Object to test.
    :return: True if string, false otherwise.
    :rtype: bool
    """
	try:  # basestring is available in python 2 but missing in python 3!
		return isinstance(obj, basestring)
	except NameError:
		return isinstance(obj, str)


def is_full_string(string):
	"""
    Check if a string is not empty (it must contains at least one non space character).

    :param string: String to check.
    :type string: str
    :return: True if not empty, false otherwise.
    """
	return is_string(string) and string.strip() != ''


# Full url example:
# scheme://username:password@www.domain.com:8042/folder/subfolder/file.extension?param=value&param2=value2#hash
def is_url(string, allowed_schemes=None):
	"""
    Check if a string is a valid url.

    :param string: String to check.
    :param allowed_schemes: List of valid schemes ('http', 'https', 'ftp'...). Default to None (any scheme is valid).
    :return: True if url, false otherwise
    :rtype: bool
    """
	if not is_full_string(string):
		return False
	valid = bool(URL_RE.search(string))
	if allowed_schemes:
		return valid and any([string.startswith(s) for s in allowed_schemes])
	return valid


def is_email(string):
	"""
    Check if a string is an email.

    | **IMPORTANT NOTES**:
    | By design, the implementation of this checking does not follow the specification for a valid \
    email address, but instead it's based on real world cases in order to match more than 99% \
    of emails and catch user mistakes. For example the percentage sign "%" is a valid sign for an email, \
    but actually no one use it, instead if such sign is found in a string coming from user input (like a \
    web form) is very likely that the intention was to type "5" (which is on the same key on a US keyboard).

    | You can take a look at "**IsEmailTestCase**" in tests.py for further details.


    :param string: String to check.
    :type string: str
    :return: True if email, false otherwise.
    :rtype: bool
    """
	return is_full_string(string) and bool(EMAIL_RE.search(string))


def is_credit_card(string, card_type=None):
	"""
    Checks if a string is a valid credit card number.
    If card type is provided then it checks that specific type,
    otherwise any known credit card number will be accepted.

    :param string: String to check.
    :type string: str
    :param card_type: Card type.
    :type card_type: str

    Can be one of these:

    * VISA
    * MASTERCARD
    * AMERICAN_EXPRESS
    * DINERS_CLUB
    * DISCOVER
    * JCB

    or None. Default to None (any card).

    :return: True if credit card, false otherwise.
    :rtype: bool
    """
	if not is_full_string(string):
		return False
	if card_type:
		if card_type not in CREDIT_CARDS:
			raise KeyError(
				'Invalid card type "{}". Valid types are: {}'.format(card_type, ', '.join(CREDIT_CARDS.keys()))
			)
		return bool(CREDIT_CARDS[card_type].search(string))
	for c in CREDIT_CARDS:
		if CREDIT_CARDS[c].search(string):
			return True
	return False


def is_camel_case(string):
	"""
    Checks if a string is formatted as camel case.
    A string is considered camel case when:

    - it's composed only by letters ([a-zA-Z]) and optionally numbers ([0-9])
    - it contains both lowercase and uppercase letters
    - it does not start with a number


    :param string: String to test.
    :type string: str
    :return: True for a camel case string, false otherwise.
    :rtype: bool
    """
	return is_full_string(string) and bool(CAMEL_CASE_TEST_RE.search(string))


def is_snake_case(string, separator='_'):
	"""
	Checks if a string is formatted as snake case.
    A string is considered snake case when:

    * it's composed only by lowercase letters ([a-z]), underscores (or provided separator) \
    and optionally numbers ([0-9])
    * it does not start/end with an underscore (or provided separator)
    * it does not start with a number


    :param string: String to test.
    :type string: str
    :param separator: String to use as separator.
    :type separator: str
    :return: True for a snake case string, false otherwise.
    :rtype: bool
    """
	if is_full_string(string):
		re_map = {
			'_': SNAKE_CASE_TEST_RE,
			'-': SNAKE_CASE_TEST_DASH_RE
		}
		re_template = '^[a-z]+([a-z\d]+{sign}|{sign}[a-z\d]+)+[a-z\d]+$'
		r = re_map.get(separator, re.compile(re_template.format(sign=re.escape(separator))))
		return bool(r.search(string))
	return False


def is_json(string):
	"""
    Check if a string is a valid json.

    :param string: String to check.
    :type string: str
    :return: True if json, false otherwise
    :rtype: bool
    """
	if not is_full_string(string):
		return False
	if bool(JSON_WRAPPER_RE.search(string)):
		try:
			return isinstance(json.loads(string), dict)
		except (TypeError, ValueError, OverflowError):
			return False
	return False


def is_uuid(string):
	"""
    Check if a string is a valid UUID.

    :param string: String to check.
    :type string: str
    :return: True if UUID, false otherwise
    :rtype: bool
    """
	return bool(UUID_RE.search(str(string)))


def is_ip(string):
	"""
    Checks if a string is a valid ip.

    :param string: String to check.
    :type string: str
    :return: True if an ip, false otherwise.
    :rtype: bool
    """
	return is_full_string(string) and bool(IP_RE.search(string))


def is_palindrome(string, strict=True):
	"""
    Checks if the string is a palindrome (https://en.wikipedia.org/wiki/Palindrome).

    :param string: String to check.
    :type string: str
    :param strict: True if white spaces matter (default), false otherwise.
    :type strict: bool
    :return: True if the string is a palindrome (like "otto", or "i topi non avevano nipoti" if strict=False),
    False otherwise
    """
	if is_full_string(string):
		if strict:
			return reverse(string) == string
		return is_palindrome(SPACES_RE.sub('', string))
	return False


def is_pangram(string):
	"""
    Checks if the string is a pangram (https://en.wikipedia.org/wiki/Pangram).

    :param string: String to check.
    :type string: str
    :return: True if the string is a pangram, False otherwise.
    """
	return is_full_string(string) and set(SPACES_RE.sub('', string)).issuperset(letters_set)


def is_isogram(string):
	"""
    Checks if the string is an isogram (https://en.wikipedia.org/wiki/Isogram).

    :param string: String to check.
    :type string: str
    :return: True if isogram, false otherwise.
    """
	return is_full_string(string) and len(set(string)) == len(string)


def is_slug(string, sign='-'):
	"""
    Checks if a given string is a slug.

    :param string: String to check.
    :type string: str
    :param sign: Join sign used by the slug.
    :type sign: str
    :return: True if slug, false otherwise.
    """
	if not is_full_string(string):
		return False
	rex = r'^([a-z\d]+' + re.escape(sign) + r'?)*[a-z\d]$'
	return re.match(rex, string) is not None


def words_count(string):
	"""
    Returns the number of words contained into the given string.

    This method is smart, it does consider only sequence of one or more letter and/or numbers
    as "words", so a string like this: "! @ # % ... []" will return zero!
    Moreover it is aware of punctuation, so the count for a string like "one,two,three.stop"
    will be 4 not 1 (even if there are no spaces in the string).

    :param string: String to check.
    :type string: str
    :return: Number of words.
    :rtype: int
    """
	return len(WORDS_COUNT_RE.findall(string))


def contains_html(string):
	"""
    Checks if the given string contains html code.
    By design, this function is very permissive regarding what to consider html code, don't expect to use it
    as an html validator, its goal is to detect "malicious" or undesired html tags in the text.

    :param string: Text to check
    :type string: str
    :return: True if string contains html, false otherwise.
    :rtype: bool
    """
	return bool(HTML_RE.search(string))


# string manipulation functions

def reverse(string):
	"""
    Returns the string reversed ("abc" -> "cba").

    :param string: String to revert.
    :type string: str
    :return: Reversed string.
    :rtype: str
    """
	return string[::-1]


def camel_case_to_snake(string, separator='_'):
	"""
    Convert a camel case string into a snake case one.
    (The original string is returned if is not a valid camel case string)

    :param string: String to convert.
    :type string: str
    :param separator: Sign to use as separator.
    :type separator: str
    :return: Converted string.
    :rtype: str
    """
	if not is_string(string):
		raise TypeError('Expected string')
	if not is_camel_case(string):
		return string
	return CAMEL_CASE_REPLACE_RE.sub(lambda m: m.group(1) + separator, string).lower()


def snake_case_to_camel(string, upper_case_first=True, separator='_'):
	"""
    Convert a snake case string into a camel case one.
    (The original string is returned if is not a valid snake case string)

    :param string: String to convert.
    :type string: str
    :param upper_case_first: True to turn the first letter into uppercase (default).
    :type upper_case_first: bool
    :param separator: Sign to use as separator (default to "_").
    :type separator: str
    :return: Converted string
    :rtype: str
    """
	if not is_string(string):
		raise TypeError('Expected string')
	if not is_snake_case(string, separator):
		return string
	re_map = {
		'_': SNAKE_CASE_REPLACE_RE,
		'-': SNAKE_CASE_REPLACE_DASH_RE
	}
	r = re_map.get(separator, re.compile('({sign})([a-z\d])'.format(sign=re.escape(separator))))
	string = r.sub(lambda m: m.group(2).upper(), string)
	if upper_case_first:
		return string[0].upper() + string[1:]
	return string


def uuid():
	"""
    Generated an UUID string (using uuid.uuid4()).

    :return: uuid string.
    :rtype: str
    """
	return str(uuid4())


def shuffle(string):
	"""
    Return a new string containing shuffled items.

    :param string: String to shuffle
    :type string: str
    :return: Shuffled string
    :rtype: str
    """
	s = sorted(string)  # turn the string into a list of chars
	random.shuffle(s)  # shuffle the list
	return ''.join(s)  # convert the shuffled list back to string


def strip_html(string, keep_tag_content=False):
	"""
    Remove html code contained into the given string.

    :param string: String to manipulate.
    :type string: str
    :param keep_tag_content: True to preserve tag content, False to remove tag and its content too (default).
    :type keep_tag_content: bool
    :return: String with html removed.
    :rtype: str
    """
	r = HTML_TAG_ONLY_RE if keep_tag_content else HTML_RE
	return r.sub('', string)


def prettify(string):
	"""
    Turns an ugly text string into a beautiful one by applying a regex pipeline which ensures the following:

    - String cannot start or end with spaces
    - String cannot have multiple sequential spaces, empty lines or punctuation (except for "?", "!" and ".")
    - Arithmetic operators (+, -, /, \*, =) must have one, and only one space before and after themselves
    - The first letter after a dot, an exclamation or a question mark must be uppercase
    - One, and only one space should follow a dot, an exclamation or a question mark
    - Text inside double quotes cannot start or end with spaces, but one, and only one space must come first and \
    after quotes (foo" bar"baz -> foo "bar" baz)
    - Text inside round brackets cannot start or end with spaces, but one, and only one space must come first and \
    after brackets ("foo(bar )baz" -> "foo (bar) baz")
    - Percentage sign ("%") cannot be preceded by a space if there is a number before ("100 %" -> "100%")
    - Saxon genitive is correct ("Dave' s dog" -> "Dave's dog")


    :param string: String to manipulate
    :return: Prettified string.
    :rtype: str
    """
	
	def remove_duplicates(regex_match):
		return regex_match.group(1)[0]
	
	def uppercase_first_letter_after_sign(regex_match):
		match = regex_match.group(1)
		return match[:-1] + match[2].upper()
	
	def ensure_right_space_only(regex_match):
		return regex_match.group(1).strip() + ' '
	
	def ensure_left_space_only(regex_match):
		return ' ' + regex_match.group(1).strip()
	
	def ensure_spaces_around(regex_match):
		return ' ' + regex_match.group(1).strip() + ' '
	
	def remove_internal_spaces(regex_match):
		return regex_match.group(1).strip()
	
	def fix_saxon_genitive(regex_match):
		return regex_match.group(1).replace(' ', '') + ' '
	
	p = PRETTIFY_RE['DUPLICATES'].sub(remove_duplicates, string)
	p = PRETTIFY_RE['RIGHT_SPACE'].sub(ensure_right_space_only, p)
	p = PRETTIFY_RE['LEFT_SPACE'].sub(ensure_left_space_only, p)
	p = PRETTIFY_RE['SPACES_AROUND'].sub(ensure_spaces_around, p)
	p = PRETTIFY_RE['SPACES_INSIDE'].sub(remove_internal_spaces, p)
	p = PRETTIFY_RE['UPPERCASE_AFTER_SIGN'].sub(uppercase_first_letter_after_sign, p)
	p = PRETTIFY_RE['SAXON_GENITIVE'].sub(fix_saxon_genitive, p)
	p = p.strip()
	try:
		return p[0].capitalize() + p[1:]
	except IndexError:
		return p


def slugify(string, sign='-'):
	"""
    Converts a string into a slug using provided join sign.
    (**(This Is A "Test"!)** -> **this-is-a-test**)

    :param string: String to convert.
    :type string: str
    :param sign: Sign used to join string tokens (default to "-").
    :type sign: str
    :return: Slugified string
    """
	if not is_string(string):
		raise TypeError('Expected string')
	
	# unicode casting for python 2 (unicode is default for python 3)
	try:
		string = unicode(string, 'utf-8')
	except NameError:
		pass
	
	# replace any character that is NOT letter or number with spaces
	s = NO_LETTERS_OR_NUMBERS_RE.sub(' ', string.lower()).strip()
	
	# replace spaces with join sign
	s = SPACES_RE.sub(sign, s)
	
	# normalize joins (remove duplicates)
	s = re.sub(re.escape(sign) + r'+', sign, s)
	
	# translate non-ascii signs
	s = unicodedata.normalize('NFD', s).encode('ascii', 'ignore').decode('utf-8')
	
	return s


def random_colors(use_hastag=False):
	"""
	Generate random colors hexa RGB
	:return Hexa color as string:
	"""
	a = hex(random.randrange(0, 256))
	b = hex(random.randrange(0, 256))
	c = hex(random.randrange(0, 256))
	a = a[2:]
	b = b[2:]
	c = c[2:]
	if len(a) < 2:
		a = "0" + a
	if len(b) < 2:
		b = "0" + b
	if len(c) < 2:
		c = "0" + c
	z = a + b + c
	if use_hastag:
		return "#" + z.lower()
	else:
		return z.lower()


def get_name_abbreviations(name=''):
	split = name.split(' ')
	result = ''
	for i, data in enumerate(split):
		if i <= 2:
			result += data[0:1].upper()
	return result


def is_blank(string):
	string = str(string)
	return not (string and string.strip())


def is_empty(string):
	"""
	:param string:
	:return True if is empty string otherwise False
	"""
	return string == '' or len(string) == 0


def is_not_empty(string):
	"""
	Check if string is not empty
	:param string:
	:return True if is non empty string otherwise False
	"""
	return bool(string)


def is_not_blank(string):
	return bool(string and string.strip())


def is_digit(string):
	return str(string).replace('.', '').isdigit()

def is_not_digit(string):
	return not (is_digit(string))

def to_camel_case(s):
	return s[0].upper() + string.capwords(s, sep='_').replace('_', '')[1:] if s else s


from visitor import AnalysisNodeVisitor
from slerp import is_json, to_camel_case
from apigen import render_template, append, write
import ast
import json
import os

def main():
	file_path = raw_input('Where is your api file? ')
	buf = open(file_path, 'r')
	file_name = os.path.basename(file_path)
	file_name = file_name.replace('_api.py', '')
	class_name = 'Test'+to_camel_case(file_name)
	service_file = 'test_' + file_name + '.py'
	if not os.path.isdir('tests'):
		os.makedirs('tests')
		write('tests/'+service_file, render_template('test_base.txt', {'class_name': class_name}))
		write('tests/__init__.py', '')
	else:
		if not os.path.isfile(service_file):
			write('tests/'+ service_file, render_template('test_base.txt', {'class_name': class_name}))
	tree = ast.parse(buf.read())
	buf.close()
	visitor = AnalysisNodeVisitor()
	visitor.visit(tree)
	
	for func in visitor.functions:
		docstring = func.attributes['docstring']
		if not is_json(docstring):
			continue
		doc_json = json.loads(docstring, encoding='UTF-8')
		func.attributes['docstring'] = doc_json
		append('tests/'+service_file,render_template('test_api.txt', func.attributes))
	pass


if __name__ == '__main__':
	# /home/kiditz/slerp-rest-api/app/api/learning_material_api.py
	main()


from flask import Response, jsonify
from .logger import logging

log = logging.getLogger(__name__)


class JsonResponse(Response):
	default_mimetype = 'application/json'
	
	@classmethod
	def force_type(cls, rv, environ=None):
		if isinstance(rv, dict):
			data = {}
			if 'status' not in rv:
				data.update({'status': 'OK'})
			else:
				data.update({'status': rv['status']})
			if 'message' in rv:
				data.update({'message': rv['message']})
			if 'key' in rv:
				data.update({'key': rv['key']})
			if 'total' in rv:
				data.update({'total': rv['total']})
				rv.pop('total')
			if 'total_pages' in rv:
				data.update({'total_pages': rv['total_pages']})
				rv.pop('total_pages')
			if 'payload' in rv:
				data.update({'payload': rv['payload']})
			
			return super(JsonResponse, cls).force_type(jsonify(data), environ)
		return super(JsonResponse, cls).force_type(rv, environ)


from kafka import KafkaProducer
from .app import app
import json


def send_message(topic, domain, partition=0):
	producer = KafkaProducer(bootstrap_servers=app.config['KAFKA_SERVER'],
	                         value_serializer=lambda v: json.dumps(v).encode('utf-8'))
	future = producer.send(topic, domain, partition=partition)
	try:
		future.get(timeout=app.config['KAFKA_TIME_OUT'])
	except Exception as e:
		app.logger.error(e, exc_info=True)



import ast
import getpass
import os
import re

from jinja2 import Environment, FileSystemLoader

from .visitor import AnalysisNodeVisitor, ClassVisitor

PATH = os.path.dirname(os.path.abspath(__file__))
# Get the root path
PATH = os.path.abspath(os.path.join(PATH, os.pardir))
print('Path %s', PATH)
TEMPLATE_ENVIRONMENT = Environment(
	autoescape=False,
	loader=FileSystemLoader(os.path.join(PATH, 'templates')),
	trim_blocks=False)


def render_template(template_filename, context):
	return TEMPLATE_ENVIRONMENT.get_template(template_filename).render(context)

def get_field_name(fields=None):
	field_names = [];
	for d in fields:
		field_names.append(d['name'])
	return field_names

# noinspection PyTypeChecker
def get_fields(key=None, classes=None):
	for clazz in classes:
		if key in clazz:
			return clazz[key]['fields']
	raise RuntimeError('not found')


def get_class_name(classes=None):
	class_name = []
	for clazz in classes:
		class_name.append([str(x) for x in clazz.keys()])
		pass
	return class_name


def to_snake_case(name):
	s1 = re.sub('(.)([A-Z][a-z]+)', r'\1_\2', name)
	return re.sub('([a-z0-9])([A-Z])', r'\1_\2', s1).lower()


def write(file_name=None, content=None):
	f = open(file_name, 'w')
	f.write(content)
	f.close()


def append(file_name=None, content=None):
	f = open(file_name, 'a')
	f.write(content)
	f.close()


def is_function_not_exists(file_name, attrib_name):
	tree = ast.parse(open(file_name, 'r').read(), '')
	visitor = AnalysisNodeVisitor()
	visitor.visit(tree)
	nodes = visitor.functions
	for node in nodes:
		if node.attributes['name'] == attrib_name:
			return False
	return True


def generate_find(domain=None, class_name=None, classes=None, service_file=None, api_file=None, filename=None):
	domain['method_name'] = to_snake_case('Find' + class_name)
	print('You have {}, \nnow please input one of choice in the filter '.format(
		str(get_field_name(domain['fields'])).replace('[', '(').replace(']', ')')))	
	filter_by = input('Filter By : ')
	filter_by_list = []
	service_name = ''
	if ',' in filter_by:
		split = filter_by.split(',')
		for filter_key in split:
			if filter_key.strip() not in get_field_name(domain['fields']):
				print('You are not lucky, your model does not have {}'.format(filter_key.strip()))
				classes[:] = []
				generate(filename)
				return
			else:
				filter_by_list.append(filter_key.strip())
				service_name = domain['method_name'] + '_by_' + '_and_'.join(filter_by_list)
	else:
		if filter_by not in get_field_name(domain['fields']):
			filter_by = ''
		else:
			service_name = domain['method_name'] + '_by_' + filter_by
	domain['filter_by'] = filter_by
	domain['filter_by_list'] = filter_by_list
	if is_function_not_exists(service_file, service_name):
		append(service_file, render_template('service_find.txt', domain))
		append(api_file, render_template('api_find.txt', domain))
		print('Successfully generated module in {}...'.format(service_file))
	else:
		print('You are not lucky, {} has been exist'.format(domain['method_name']))
		print('Lets try again\n')
		classes[:] = []
		generate(filename)


def generate_create(domain=None, class_name=None, classes=None, service_file=None, api_file=None, filename=None):
	domain['method_name'] = to_snake_case('Add' + class_name)
	if is_function_not_exists(service_file, domain['method_name']):
		append(service_file, render_template('service_add.txt', domain))
		append(api_file, render_template('api_add.txt', domain))
		print('Successfully generated module in {}...'.format(service_file))
	else:
		print('You are not lucky, {} has been exist'.format(domain['method_name']))
		classes[:] = []
		generate(filename)


def generate_delete(domain=None, class_name=None, classes=None, service_file=None, api_file=None, filename=None):
	domain['method_name'] = to_snake_case('Delete' + class_name)
	print('You have {}, \nNow select one of all the options you have'.format(
		str(get_field_name(domain['fields'])).replace('[', '(').replace(']', ')')))	
	filter_by = input('Delete By : ')
	filter_by_list = []
	service_name = ''
	if ',' in filter_by:
		split = filter_by.split(',')
		for filter_key in split:
			if filter_key.strip() not in get_field_name(domain['fields']):
				print('You are not lucky, your model does not have {}'.format(filter_key.strip()))
				classes[:] = []
				generate(filename)
				return
			else:
				filter_by_list.append(filter_key.strip())
				service_name = domain['method_name'] + '_by_' + '_and_'.join(filter_by_list)
	else:
		if filter_by not in get_field_name(domain['fields']):
			filter_by = ''
		else:
			service_name = domain['method_name'] + '_by_' + filter_by
	domain['filter_by'] = filter_by
	domain['filter_by_list'] = filter_by_list
	if is_function_not_exists(service_file, service_name):
		append(service_file, render_template('service_delete.txt', domain))
		append(api_file, render_template('api_delete.txt', domain))
		print('Successfully generated module in {}...'.format(service_file))
	else:
		print('You are not lucky, {} has been exist'.format(domain['method_name']))
		print('Lets try again\n')
		classes[:] = []
		generate(filename)


def generate_update(domain=None, class_name=None, classes=None, service_file=None, api_file=None, filename=None):
	domain['method_name'] = to_snake_case('Edit' + class_name)
	print ('You have {}, \nnow please input one of choice in the filter '.format(
		str(get_field_name(domain['fields'])).replace('[', '(').replace(']', ')')))	
	filter_by = input('Update By : ')
	filter_by_list = []
	service_name = ''
	if ',' in filter_by:
		split = filter_by.split(',')
		for filter_key in split:
			if filter_key.strip() not in get_field_name(domain['fields']):
				print('You are not lucky, your model does not have {}'.format(filter_key.strip()))
				classes[:] = []
				generate(filename)
				return
			else:
				filter_by_list.append(filter_key.strip())
				service_name = domain['method_name'] + '_by_' + '_and_'.join(filter_by_list)			
		pass
	else:
		if filter_by not in get_field_name(domain['fields']):
			filter_by = ''
		else:
			service_name = domain['method_name'] + '_by_' + filter_by

	domain['filter_by'] = filter_by
	domain['filter_by_list'] = filter_by_list
	if is_function_not_exists(service_file, service_name):
		append(service_file, render_template('service_update.txt', domain))
		append(api_file, render_template('api_update.txt', domain))
		print('Successfully generated module in {}...'.format(service_file))
	else:
		print('You are not lucky, {} has been exist'.format(service_name))
		print('Lets try again\n')
		classes[:] = []
		generate(filename)


def generate_paging(domain=None, class_name=None, classes=None, service_file=None, api_file=None, filename=None):
	domain['method_name'] = to_snake_case('Get' + class_name)
	print('You have {}, \nNow select one of all the options you have'.format(
		str(get_field_name(domain['fields'])).replace('[', '(').replace(']', ')')))	
	filter_by = input('Filter By :')
	filter_by_list = []
	service_name = ''
	if ',' in filter_by:
		split = filter_by.split(',')
		
		for filter_key in split:
			if filter_key.strip() not in get_field_name(domain['fields']):
				print('You are not lucky, your model does not have {}'.format(filter_key.strip()))
				classes[:] = []
				generate(file)
				return
			else:
				filter_by_list.append(filter_key.strip())
				service_name = domain['method_name'] + '_by_' + '_and_'.join(filter_by_list)
	
	else:
		if filter_by not in get_field_name(domain['fields']):
			print('You are not lucky, your model does not have {}'.format(filter_by))
			classes[:] = []
			generate(filename)
		else:
			service_name = domain['method_name'] + '_by_' + filter_by
	
	order_by = str(input('Order By : ')).strip()	
	domain['order_type'] = 'asc'
	is_asc = input('Is asc order? ')
	if is_asc.lower() == 'yes':
		domain['order_type'] = 'asc()'
	else:
		domain['order_type'] = 'desc()'
	
	if order_by not in get_field_name(domain['fields']):
		print('You are not lucky, your model does not have '.format(order_by))
		print('Lets try again\n')
		classes[:] = []
		generate(filename)
	domain['order_by'] = order_by
	domain['filter_by'] = filter_by
	domain['filter_by_list'] = filter_by_list
	if is_function_not_exists(service_file, service_name):
		append(service_file, render_template('service_pagging.txt', domain))
		append(api_file, render_template('api_pagging.txt', domain))
		print('Successfully generated module in {}...'.format(service_file))
	else:
		print('You are not lucky, {} has been exist'.format(service_name))
		print('Lets try again\n')
		classes[:] = []
		generate(filename)


def generate_all_item(domain=None, class_name=None, classes=None, service_file=None, api_file=None, filename=None):
	domain['method_name'] = to_snake_case('Get' + class_name)
	print('You have {}, \nNow select one of all the options you have'.format(
		str(get_field_name(domain['fields'])).replace('[', '(').replace(']', ')')))	
	filter_by = input('Filter by : ')
	filter_by_list = []
	service_name = ''
	if ',' in filter_by:
		split = filter_by.split(',')
		
		for filter_key in split:
			if filter_key.strip() not in get_field_name(domain['fields']):
				print('You are not lucky, your model does not have {}'.format(filter_key.strip()))
				classes[:] = []
				generate(file)
				return
			else:
				filter_by_list.append(filter_key.strip())
				service_name = domain['method_name'] + '_by_' + '_and_'.join(filter_by_list)
	
	else:
		if filter_by not in get_field_name(domain['fields']):
			print('You are not lucky, your model does not have {}'.format(filter_by))
			classes[:] = []
			generate(filename)
		else:
			service_name = domain['method_name'] + '_by_' + filter_by
	
	
	order_by = str(input('Order by : ')).strip()	
	domain['order_type'] = 'asc'
	is_asc = input('Is this ascending (yes/no) ? ')
	if is_asc.lower() == 'yes':
		domain['order_type'] = 'asc()'
	else:
		domain['order_type'] = 'desc()'
	
	if order_by not in get_field_name(domain['fields']):
		print('You are not lucky, your model does not have '.format(order_by))
		print('Lets try again\n')
		classes[:] = []
		generate(filename)
	domain['order_by'] = order_by
	domain['filter_by'] = filter_by
	domain['filter_by_list'] = filter_by_list
	if is_function_not_exists(service_file, service_name):
		append(service_file, render_template('service_all.txt', domain))
		append(api_file, render_template('api_all.txt', domain))
		print('Successfully generated module in {}...'.format(class_name))
	else:
		print('You are not lucky, {} has been exist'.format(service_name))
		print('Lets try again\n')
		classes[:] = []
		generate(filename)


def generate(filename=None):
	domain = {}
	buf = open(filename, 'r')
	tree = ast.parse(buf.read(), '')
	buf.close()
	visitor = ClassVisitor()
	visitor.visit(tree)
	classes = visitor.get_classes
	print('All right, is this your models file.')
	print ('***************************************************************************** ')
	classes_names = '\n'.join(map(str, [i[0] for i in get_class_name(classes)]))
	print (classes_names)
	print ('***************************************************************************** ')	
	class_name = input('So please select one that you need ? ')		
	q_type = str(input('Okay now, what is query you want to add(c, r, u, or d) ? ')).lower()
	
	domain['class_name_snake'] = to_snake_case(class_name)
	domain['class_name'] = class_name
	service_file = 'service/{}_service.py'.format(domain['class_name_snake'])
	api_file = 'api/{}_api.py'.format(domain['class_name_snake'])
	if not os.path.isdir('service'):
		os.makedirs('service')
		write(service_file, render_template('service_base.txt', domain))
		write('service/__init__.py', '')
	else:
		if not os.path.isfile(service_file):
			write(service_file, render_template('service_base.txt', domain))
	
	if not os.path.isdir('api'):
		os.makedirs('api')
		write(api_file, render_template('api_base.txt', domain))
		write('api/__init__.py', '')
	else:
		if not os.path.isfile(api_file):
			write(api_file, render_template('api_base.txt', domain))
	domain['fields'] = get_fields(key=class_name, classes=classes)
	if q_type == 'c':
		generate_create(domain=domain, class_name=class_name, classes=classes, service_file=service_file,
		                api_file=api_file,
		                filename=filename)
	elif q_type == 'u':
		generate_update(domain=domain, class_name=class_name, classes=classes, service_file=service_file,
		                api_file=api_file,
		                filename=filename)
	elif q_type == 'r':		
		is_paging = str(input('Do you want pagination (yes/no) ? ')).lower()
		paging = True if is_paging == 'yes' else False
		if paging:
			generate_paging(domain=domain, class_name=class_name, classes=classes, service_file=service_file,
			                api_file=api_file,
			                filename=filename)
			return		
		is_one_item = str(input('is this just one item (yes/no) ?')).lower()
		one_item = True if is_one_item == 'yes' else False
		if one_item:
			generate_find(domain=domain, class_name=class_name, classes=classes, service_file=service_file,
			              api_file=api_file,
			              filename=filename)
			return
		
		is_all_item = str(input('is have all item (yes/no) ?')).lower()
		
		all_item = True if is_all_item == 'yes' else False
		
		if all_item:
			generate_all_item(domain=domain, class_name=class_name, classes=classes, service_file=service_file,
			                  api_file=api_file,
			                  filename=filename)
		else:
			print('sorry i dont know what you want, bye...')
			exit(0)
	elif q_type == 'd':
		generate_delete(domain=domain, class_name=class_name, classes=classes, service_file=service_file,
		                api_file=api_file,
		                filename=filename)
	else:
		print('You are not lucky, please choose correctly')
		generate(filename)


def main():
	print('\n\nHello {}, my name is Typo.\n'
	      'i can help you to create flask api code\nbut you need to answer my question first.'.format(
		getpass.getuser()))
	print('***************************************************************************** ')	
	file_name = str(input('Where is your model file ? '))
	if not os.path.isfile(file_name):
		print('Invalid file path, try again.')		
		file_name = str(input('Where is your model file ? '))
		generate(file_name)
	else:
		generate(file_name)

if __name__ == '__main__':
	main()


from requests.exceptions import ConnectionError


class ValidationException(Exception):
	"""Custom ValidationException For """
	
	def __init__(self, message, key=None):
		self.message = message
		self.key = key
		super(ValidationException, self).__init__(message)


class CoreException(Exception):
	"""Custom ValidationException For """
	
	def __init__(self, message):
		self.message = message
		super(CoreException, self).__init__(message)


class ConsulException(ConnectionError):
	pass



#!/usr/bin/env python
"""
slerp-framework
-------------
Framework to handle simplify development application with,
flask, consulate and sql alchemy together


"""

import sys

from setuptools import setup, find_packages


# Dependencies:

if sys.version_info[0] == 2:
    dnspython = 'dnspython'
elif sys.version_info[0] == 3:
    dnspython = 'dnspython3'
else:
    raise ValueError('Unsupported python version')

INSTALL_REQUIRES = [
    'consulate',
    'Flask',
    'Flask-SQLAlchemy',
    'dnspython',
    'kafka-python'
]

TESTS_REQUIRE = [
    'httpretty',
    'mock',
    'six',
    'coveralls',
    'coverage',
    'pytest',
]


setup(
    name='slerp-py',
    version='0.0.14',
    url='https://github.com/kiditz/slerpio',
    license='Apache',
    author='Rifky Aditya Bastara',
    author_email='kiditzbastara@gmail.com',

    description='support library for slerp project',
    long_description=__doc__,
    packages=find_packages(),
    zip_safe=False,
    include_package_data=True,
    platforms='any',    
    install_requires=INSTALL_REQUIRES,
    tests_require=TESTS_REQUIRE,
    extras_require={
        'test': TESTS_REQUIRE,
    },
    test_suite='tests',

    classifiers=[
        'Development Status :: 1 - Planning',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
    entry_points={
        'console_scripts': [
            'apigen = slerp.apigen:main',
            'testgen = slerp.testgen:main'
        ],
    }
)



import ast
import os


def _get_last_line_number(nodes):
	children = None
	if hasattr(nodes[-1], 'orelse'):
		children = nodes[-1].orelse
	elif hasattr(nodes[-1], 'finalbody'):
		children = nodes[-1].finalbody
	elif hasattr(nodes[-1], 'body'):
		children = nodes[-1].body
	if children:
		return max(nodes[-1].lineno, _get_last_line_number(children))
	else:
		return nodes[-1].lineno


class Node(object):
	
	def __init__(self, attributes={}, parent=None):
		self.__dict__.update({'_children': [], '_parent': None, '_attributes': attributes})
		self.parent = parent
	
	def __repr__(self):
		return self.__class__.__name__ + "(attributes = " + str(self.attributes) + ")"
	
	@property
	def attributes(self):
		return self._attributes
	
	@attributes.setter
	def attributes(self, attributes):
		self._attributes = attributes
	
	@property
	def parent(self):
		return self._parent
	
	@parent.setter
	def parent(self, parent):
		if self._parent != None:
			self._parent.removeChild(self)
		self._parent = parent
		if self._parent != None:
			self._parent.addChild(self)
	
	@property
	def children(self):
		return self._children
	
	def removeChild(self, child):
		if child in self.children():
			del self._children[self._children.indexof(child)]
	
	def addChild(self, child):
		if not child in self._children:
			self._children.append(child)


class AnalysisNodeVisitor(ast.NodeVisitor):
	
	def __init__(self, rootNode=None):
		self._modules = []
		self._classes = []
		self._functions = []
		self._variables = []
		self._imports = []
		self._rootNode = rootNode
		self._parentNode = rootNode
		self._level = 0
	
	@property
	def rootNode(self):
		return self._rootNode
	
	@property
	def imports(self):
		return self._imports
	
	@property
	def functions(self):
		return self._functions
	
	@property
	def variables(self):
		return self._variables
	
	@property
	def classes(self):
		return self._classes
	
	def visit_Import(self, node):
		for name in node.names:
			importNode = Node(attributes={'type': 'import', 'names': map(lambda x: x.name, node.names)},
			                  parent=self._parentNode)
			self._imports.append(importNode)
		ast.NodeVisitor.generic_visit(self, node)
	
	def visit_ImportFrom(self, node):
		for name in node.names:
			importNode = Node(attributes={'line_number': node.lineno, 'type': 'from_import', 'module': node.module,
			                              'names': map(lambda x: x.name, node.names)}, parent=self._parentNode)
			self._imports.append(importNode)
		ast.NodeVisitor.generic_visit(self, node)
	
	def visit_Assign(self, node):
		for target in node.targets:
			self._add_target_to_variables(target)
		ast.NodeVisitor.generic_visit(self, node)
	
	def visit_AssignAug(self, node):
		self._add_target_to_variables(node.target)
		ast.NodeVisitor.generic_visit(self, node)
	
	def _add_target_to_variables(self, target):
		if hasattr(target, 'value'):
			self._add_target_to_variables(target.value)
		elif hasattr(target, 'id'):
			if not target.id in self._variables and not target.id == "self":
				variableNode = Node(attributes={'type': 'variable', 'name': target.id}, parent=self._parentNode)
				self._variables.append(variableNode)
	
	def visit_FunctionDef(self, node):
		body = node.body
		http_method = ''
		try:
			if hasattr(node, 'decorator_list'):
				http_method = node.decorator_list[0].keywords[0].value.elts[0].s
		except Exception:
			pass
		functionNode = Node(attributes={'type': 'function', 'name': node.name, 'start_line': body[0].lineno,
		                                'end_line': _get_last_line_number(body), 'docstring': ast.get_docstring(node),
		                                'http': http_method},
		                    parent=self._parentNode)
		self._functions.append(functionNode)
		
		oldParent = self._parentNode
		self._parentNode = functionNode
		ast.NodeVisitor.generic_visit(self, node)
		self._parentNode = oldParent
	
	def visit_ClassDef(self, node):
		body = node.body
		classNode = Node(attributes={'type': 'class', 'name': node.name, 'start_line': body[0].lineno,
		                             'end_line': _get_last_line_number(body), 'docstring': ast.get_docstring(node)},
		                 parent=self._parentNode)
		self._classes.append(classNode)
		oldParent = self._parentNode
		self._parentNode = classNode
		ast.NodeVisitor.generic_visit(self, node)
		self._parentNode = oldParent


class ClassVisitor(ast.NodeVisitor):
	_classes = []
	
	def visit_ClassDef(self, node):
		body = node.body
		fields = []
		for statement in node.body:
			if isinstance(statement, ast.Assign):
				if len(statement.targets) == 1 and isinstance(statement.targets[0], ast.Name):
					stmt = str(statement.targets[0].id)
					if not (stmt.startswith('__')) and stmt != 'created_at' and stmt != 'update_at':
						if isinstance(statement.value, ast.Call):
							data = ast.dump(statement.value)
							if 'BigInteger' in data:
								data_type = 'Long'
							elif 'String' in data:
								data_type = 'String'
							elif 'DateTime' in data:
								data_type = 'Date'
							elif 'Text' in data:
								data_type = 'String'
							elif 'ForeignKey' in data:
								data_type = 'Long'
							elif 'Numeric' in data:
								data_type = 'Double'
							elif 'Boolean' in data:
								data_type = 'Boolean'
							else:
								data_type = '?'
						fields.append({'name': stmt, 'type': data_type})
					pass
				pass
		clazz = {node.name: {'fields': fields, 'start_line': body[0].lineno, 'end_line': _get_last_line_number(body)}}
		self._classes.append(clazz)
	
	@property
	def get_classes(self):
		return self._classes



import os

from consulate import Session

from .decorator import with_retry_connections
from .exception import CoreException
from six import iteritems
import json


class Consulate(object):
	def __init__(self, app=None, **kwargs):
		self.kwargs = kwargs
		self.app = app
		self.host = self.kwargs.get('consul_host') or os.environ.get('CONSUL_HOST', 'localhost')
		self.port = self.kwargs.get('consul_port') or os.environ.get('CONSUL_PORT', 8500)
		self.max_tries = self.kwargs.get('max_tries', 3)
		self.session = None
		if app is not None:
			self.init_app(app)
		
	def init_app(self, app):
		self.app = app
		if not hasattr(app, 'extensions'):
			app.extensions = {}
		if 'consul' in app.extensions:
			raise CoreException('Flask application already initialized')
		self.session = self.create_session(test_connection=self.kwargs.get('test_connection', False))
		
	@with_retry_connections()
	def create_session(self, test_connection=False):
		session = Session(host=self.host, port=self.port)
		if test_connection:
			session.status.leader()
		return session
	
	@with_retry_connections()
	def register(self, **kwargs):
		kwargs.setdefault('name', self.app.name)
		self.session.agent.service.register(**kwargs)
		
	def load_config(self, namespace=None):
		if namespace is None:
			namespace = "config/{service}/{environment}/".format(
				service=os.environ.get('SERVICE', 'generic_service'),
				environment=os.environ.get('ENVIRONMENT', 'generic_environment')
			)
		
		for k, v in self.session.kv.find(namespace).items():
			k = k.replace(namespace, '')
			try:
				data = json.loads(v)
				for key, value in data.items():
					self.app.config[key] = value
				
			except (TypeError, ValueError):
				self.app.logger.info("Couldn't de-serialize {} to json, using raw value".format(v))
				self.app.config[k] = v


import json
import os
from sqlalchemy.exc import DatabaseError
from flask import Flask
from flask_sqlalchemy import SQLAlchemy
import codecs
from .consul import Consulate
from .exception import ValidationException, CoreException
from .logger import logging
from .response import JsonResponse

log = logging.getLogger(__name__)

DEFAULT_CONFIG = 'config.json'
if not os.path.exists(DEFAULT_CONFIG):
	raise RuntimeError('please create config.json file in your project root folder')


def init_apps(configuration):
	apps = Flask(__name__)
	consul = Consulate(app=apps)
	consul.load_config(namespace=configuration['namespace'])
	consul.register(name=configuration['name'], interval=configuration['interval'], httpcheck=configuration['health'], port=configuration['port'])
	return apps


json_file = open(DEFAULT_CONFIG, 'rb')
reader = codecs.getreader('utf-8')
config = json.load(reader(json_file))
json_file.close()
app = init_apps(config)
db = SQLAlchemy(app=app)
app.response_class = JsonResponse


@app.errorhandler(Exception)
# noinspection PyTypeChecker
def handle_error(error):
	log.error(error, exc_info=True)
	msg = {'status': 'FAIL'}
	if type(error) is TypeError:
		msg['message'] = 'type.error'
	elif type(error) is ValidationException:
		msg['message'] = error.message
		if error.key is not None:
			msg['key'] = error.key
			pass
	elif type(error) is CoreException:
		msg['message'] = error.message
	else:
		message = [str(x) for x in error.args]
		msg['message'] = message
	return msg


@app.after_request
def session_commit(response):
	if 'AUTO_ROLLBACK' in app.config and app.config['AUTO_ROLLBACK']:
		db.session.rollback()
	else:
		try:
			db.session.commit()
		except DatabaseError:
			db.session.rollback()
	return response


def run(**kwargs):
	kwargs.update({'port': app.config['SERVER_PORT']})
	if 'SERVER_HOST' in app.config:
		kwargs.update({'host': app.config['SERVER_HOST']})
	app.run(**kwargs)